#pragma once

// C++ system level
#include <vector>
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>
#include "Service.hpp"

// qthing stuff
#include "SiliconTorch/FxCyanRGB8.hpp"
// #include <qthing>

// misc
#include <nlohmann/json.hpp>


namespace SiliconTorch {

  namespace Service {

    class CyanBus : public ServiceManager::Service {

      public:

        void init();

        void start();


        static const u8 MAX_INTERFACES = 2;  // Limitted to the available UART HW units


        typedef struct {
          u8   tx    = 0xFF;
          u8   rx    = 0xFF;
          u8   de    = 0xFF;
          u8   re    = 0xFF;
          u32  baud  = 115200;
          bool valid = false;
        } InterfaceCfg;

        InterfaceCfg readInterfaceData(u32 number) const;

        virtual nlohmann::json getConfigJSON() const;

        // TODO: should we block copy/assignment by default…?
        CyanBus() {};
        CyanBus(const CyanBus&) = delete;
        CyanBus& operator=(CyanBus const&) = delete;

      private:

        std::vector<InterfaceCfg> ifaceCfgs;

    };

  }
}
