#include "FxCyanF.hpp"

// C++ system level
#include <cmath>
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>
#include "SiliconTorch/FxCyanF.hpp"
#include "SiliconTorch/NVSExplorer.hpp"

// qthing stuff
// #include <qthing>

// misc
#include <nlohmann/json.hpp>


using nlohmann::json;



namespace SiliconTorch {

  namespace Service {

    void FxCyanF::init() {
      setName("FxCyanF");
      setNameSpace("fxCyan");  // TODO: "FxCyanF"

      addWantedService("CyanBus");
    }


    void FxCyanF::start() {

      const char* TAG = getName().c_str();

      if (fxCyan != NULL) {
        ESP_LOGW(TAG, "Service already running!");
        return;
      }


      fxCyan = new SiliconTorch::FxCyanF::FxCyanF();

      u32 frq = SiliconTorch::FxCyanF::DefaultFrequency;
      u8  res = SiliconTorch::FxCyanF::DefaultResolution;

      NVSExplorer::NVSExplorer& nvs = NVSExplorer::NVSExplorer::instance();

      frq = nvs.getUnsignedInt(getNameSpace(), "frequency", frq);
      res = nvs.getUnsignedInt(getNameSpace(), "resolution", res);

      fxCyan->setFrqRes(frq, res);

      savedCfg["frequency"]  = fxCyan->getFrequency();
      savedCfg["resolution"] = fxCyan->getResolution();

      json channels;

      for (u8 ch = 0; ch < SiliconTorch::FxCyanF::MAX_CHANNELS; ch++) {

        char buffer[32];
        snprintf(buffer, 32, "ch%d.gpio", ch);

        u8 gpio = nvs.getUnsignedInt(getNameSpace(), buffer, 0xFF) & 0xFF;

        snprintf(buffer, 32, "ch%d.pwm", ch);
        f32 pwm = nvs.getFloat(getNameSpace(), buffer);

        // NaN == "no value stored" or any other reading error
        if (std::isnan(pwm)) pwm = 0.0f;

        if (gpio < 0xFF) {  // key exists

          if ( fxCyan->addChannel(gpio, pwm) ) {  // channel creation successful
  
            ESP_LOGI(TAG, "Configuring: FxCyanF{ channel[ %d ]  gpio[ %d ]  pwm[ %4.3f ] }", ch, gpio, pwm);
            fxCyan->setPWM(ch, pwm);

            json channel;

            channel["pwm"]  = pwm;
            channel["gpio"] = gpio;

            channels[ch] = channel;

          } else {
            ESP_LOGW(TAG, "Configuring: FxCyanF{ channel[ %d ]  gpio[ %d ] }  FAILED[❌]", ch, gpio);
            break;
          }

        } else {
          break;
        }
      }

      savedCfg["channels"] = channels;
    }


    SiliconTorch::FxCyanF::FxCyanF* FxCyanF::getFxCyanF() const {
      return fxCyan;
    }


    json FxCyanF::getConfigJSON() const {
      return savedCfg;
    }

  }
}
